<?php

namespace FiloBlu\Core\Model\Mail;

use Exception;
use FiloBlu\Core\Framework\Model\Mail\MailerInterface;
use FiloBlu\Core\Framework\Model\Mail\MessageInterface;

use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\MailException;
use Magento\Framework\Mail\Template\TransportBuilder;
use Magento\Framework\Translate\Inline\StateInterface;
use Psr\Log\LoggerInterface;
use Throwable;

/**
 * Class Mailer
 * @package FiloBlu\Core\Model\Mail
 */
class Mailer implements MailerInterface
{
    /**
     * @var StateInterface
     */
    private $inlineTranslation;

    /**
     * @var TransportBuilder
     */
    private $transportBuilder;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * Data constructor.
     * @param TransportBuilder $transportBuilder
     * @param StateInterface $inlineTranslation
     * @param LoggerInterface $logger
     */
    public function __construct(
        TransportBuilder $transportBuilder,
        StateInterface $inlineTranslation,
        LoggerInterface $logger
    ) {
        $this->transportBuilder = $transportBuilder;
        $this->inlineTranslation = $inlineTranslation;
        $this->logger = $logger;
    }

    /**
     * Send Mail
     *
     * @param MessageInterface $message
     * @return $this
     *
     * @throws LocalizedException
     * @throws MailException
     * @throws Exception
     * @throws Throwable
     */
    public function send(MessageInterface $message)
    {
        $this->inlineTranslation->suspend();
        $template = $message->getTemplate();

        $this->transportBuilder
            ->setTemplateIdentifier($template)
            ->setTemplateOptions($message->getTemplateOptions())
            ->setTemplateVars($message->getTemplateVariables());

        $this->transportBuilder->setFromByScope([
            'email' => $message->getFrom(),
            'name'  => $message->getFrom()
        ]);

        foreach ($message->getTo() as $to) {
            $this->transportBuilder->addTo($to);
        }

        foreach ($message->getBcc() as $bcc) {
            $this->transportBuilder->addBcc($bcc);
        }

        foreach ($message->getCc() as $cc) {
            $this->transportBuilder->addCc($cc);
        }

        $replyTo = $message->getReplyTo();

        if ($replyTo) {
            $this->transportBuilder->setReplyTo($replyTo);
        }

        $transport = $this->transportBuilder->getTransport();

        $lastException = null;

        try {
            $transport->sendMessage();
        } catch (Exception $exception) {
            $lastException = $exception;
            $this->logger->critical($exception->getMessage(), ['context' => $exception]);
        } catch (Throwable $throwable) {
            $lastException = $throwable;
            $this->logger->critical($throwable->getMessage(), ['context' => $throwable]);
        }

        $this->inlineTranslation->resume();

        if ($lastException) {
            throw $lastException;
        }

        return $this;
    }
}
